#include "Event.h"
#include "EventDiscriminatorRead.h"
#include "EventDiscVar.h"
#include "DiscAnalysis.h"

#include "TH1F.h"
#include "TFile.h"

#include <sstream>

using namespace std;

DiscAnalysis::DiscAnalysis() {
}


DiscAnalysis::~DiscAnalysis() {
}


void DiscAnalysis::init( const string& name ) {
  dCreate( 1,
           "+chi2,+muoKink,+trkKink,+deltaEta,+muoComp,+muoSum", name );
  dCreate( 2,
           "+chi2,+muoKink,+trkKink,-deltaEta,-muoComp,-muoSum", name );
  dCreate( 3,
           "-chi2,-muoKink,-trkKink,+deltaEta,+muoComp,+muoSum", name );
  dCreate( 4,
           "+chi2,+muoKink,-trkKink,-deltaEta,-muoComp,-muoSum", name );
  dCreate( 5,
           "-chi2,-muoKink,+trkKink,+deltaEta,-muoComp,-muoSum", name );
  dCreate( 6,
           "-chi2,-muoKink,-trkKink,-deltaEta,+muoComp,+muoSum", name );
}


void DiscAnalysis::analyze( const Event& ev ) {
  int i;
  int n = dPlot.size();
  DiscPlot* dp;
  if ( n ) {
    dp = dPlot.front();
    fill( dp, ev, dp->disc->get( ev ) );
  }
  for ( i = 1; i < n ; ++i ) {
    dp = dPlot[i];
    fill( dp, ev, dp->disc->get() );
  }
  return;
}


void DiscAnalysis::save( const string& name ) {
  TDirectory* currentDir = gDirectory;
  TFile* file = new TFile( name.c_str(), "RECREATE" );
  for ( auto dp: dPlot ) {
    dp->hSig->Write();
    dp->hBkg->Write();
  }
  file->Close();
  delete file;
  currentDir->cd();
  return;
}


void DiscAnalysis::dCreate( int id, const string& vars, const string& name ) {

  // create name for TH1F object as "h_dSig" and "h_dBkg" with X given by id
  stringstream sstr;
  sstr.clear();
  sstr.str( "" );
  sstr << "h_dSig" << id;
  string strS = sstr.str();
  const char* hNSig = strS.c_str();
  sstr.clear();
  sstr.str( "" );
  sstr << "h_dBkg" << id;
  string strB = sstr.str();
  const char* hNBkg = strB.c_str();

  // save pointer of last EventDiscriminatorRead
  static EventDiscriminatorRead* edr = nullptr;

  // create TH1F and EventDiscriminatorRead objects and store their pointers
  DiscPlot* dp = new DiscPlot;
  edr = 
  dp->disc = new EventDiscriminatorRead( edr );
  dp->hSig = new TH1F( hNSig, hNSig, 20, 0.0, 1.0 );
  dp->hBkg = new TH1F( hNBkg, hNBkg, 20, 0.0, 1.0 );
  dPlot.push_back( dp );

  // read reference distributions
  dp->disc->read( name );

  // loop over variables
  int pvar = 0;
  int llen = vars.length();
  string vtmp;
  const char* nvar;
  while ( llen ) {
    // extract variable flag and name
    string::size_type vlen = vars.find( ",", pvar );
    if ( vlen == string::npos ) {
      vlen = llen;
      nvar = vars.c_str() + pvar;
    }
    else {
      vlen -= pvar;
      vtmp = vars.substr( pvar, vlen++ );
      nvar = vtmp.c_str();
    }
    // activate or deactivate variable use according to flag
    dp->disc->setVariable( nvar + 1, *nvar == '+' );
    pvar += vlen;
    llen -= vlen;
  }

  return;

}


void DiscAnalysis::fill( DiscPlot* dp, const Event& ev, float dv ) {
  ( ev.evType() == DiscriminatorClassification::signal ?
                   dp->hSig :
                   dp->hBkg )->Fill( dv );
  return;
}

